/**
 * This script is used to dump the contents of a directory into a file.
 * Default output file is <directory>.txt
 *
 * Use it as follows:
 * npx ts-node codedump.ts <directory> <output file> <type> <topN>
 *
 * - directory: The directory to dump the contents of.
 * - output file: The file to write the contents to.
 * - type: The type of output to generate.
 *   - list: Only list file paths
 *   - normal: Include file contents with formatting (default)
 *   - verbose: Include file contents with additional metadata
 *   - minify: Include file contents with minimal whitespace
 * - topN: The number of largest files to include in the output.
 *
 * Optional flags:
 * --no-largest-files: Don't show largest files section
 * --show-largest-files: Show largest files section (default)
 *
 * Configuration:
 * You can create a codedump.config.ts file to customize the behavior.
 * This can be done through the CLI interface or manually.
 * The config file allows you to set:
 * - Default directory to dump
 * - Default output filename
 * - Default output format
 * - Whether to show largest files
 * - Files and directories to include/exclude
 *
 * Example:
 * npx ts-node codedump.ts .
 * npx ts-node codedump.ts . --type=minify
 * npx ts-node codedump.ts -m
 */
interface FileInfo {
    size: number;
    lastModified: string;
    language?: string;
    imports?: string[];
    fileType?: string;
    lineCount?: number;
}
declare function getFileInfo(filePath: string): FileInfo;
declare function shouldSkip(pathname: string): boolean;
declare function concatenateFiles(directory?: string, type?: "list" | "normal" | "verbose" | "minify"): string;
declare function getLargestFiles(directory?: string, topN?: number): [number, string][];
declare function formatSize(size: number): string;
export { getFileInfo, shouldSkip, concatenateFiles, getLargestFiles, formatSize, };
/**
 * Creates a config file with the default settings or current settings
 * @param directory The directory to use as default in the config
 * @param output The output filename to use as default in the config
 * @param type The output type to use as default in the config
 * @param showLargestFiles Whether to show largest files
 * @returns True if successful, false if failed
 */
export declare function createConfigFile(directory?: string, output?: string, type?: "list" | "normal" | "verbose" | "minify", showLargestFiles?: boolean): Promise<boolean>;
